package Net::Sident;

use strict;
use Carp;
use vars qw($VERSION @ISA @EXPORT @EXPORT_OK $AUTOLOAD);

require Exporter;
require DynaLoader;
require AutoLoader;

@ISA = qw(Exporter DynaLoader);
# Items to export into callers namespace by default. Note: do not export
# names by default without a very good reason. Use EXPORT_OK instead.
# Do not simply export all your public functions/methods/constants.
@EXPORT = qw(
	IDENT_AUTH_FAIL
	IDENT_AUTH_NOT_SUPPORTED
	IDENT_AUTH_OKAY
	IDENT_FLAG_NOT_SUPPORTED
	IDENT_HIDDEN_USER
	IDENT_INTERNAL_ERR
	IDENT_INVALID_FLAG_VALUE
	IDENT_INVALID_PORT
	IDENT_INVALID_REQ_INFO
	IDENT_INVALID_RESP_INFO
	IDENT_MAX_ERROR
	IDENT_MUTUAL_AUTH_FAIL
	IDENT_NO_MUTUAL_AUTH
	IDENT_NO_USER
	IDENT_READ_TIMEOUT
	IDENT_SYSTEM_ERROR
	IDENT_TIMEOUT
	IDENT_UNKNOWN_ERROR
	IDENT_USER_CANT_AUTH
	IDENT_USER_WONT_AUTH
	set_authtype
	get_authflag
	set_authflag
	lookup
	query
);
$VERSION = '0.2';

sub AUTOLOAD {
    # This AUTOLOAD is used to 'autoload' constants from the constant()
    # XS function.  If a constant is not found then control is passed
    # to the AUTOLOAD in AutoLoader.

    my $constname;
    ($constname = $AUTOLOAD) =~ s/.*:://;
    croak "& not defined" if $constname eq 'constant';
    my $val = constant($constname, @_ ? $_[0] : 0);
    if ($! != 0) {
	if ($! =~ /Invalid/) {
	    $AutoLoader::AUTOLOAD = $AUTOLOAD;
	    goto &AutoLoader::AUTOLOAD;
	}
	else {
		croak "Your vendor has not defined Net::Sident macro $constname";
	}
    }
    eval " sub $AUTOLOAD { $val } ";
    goto &$AUTOLOAD;
}

bootstrap Net::Sident $VERSION;

# Preloaded methods go here.

# Autoload methods go after =cut, and are processed by the autosplit program.

1;
__END__
# Below is the stub of documentation for your module. You better edit it!

=head1 NAME

Net::Sident - Perl extension for Secure Ident Protocol

=head1 SYNOPSIS


  use Net::Sident qw( /IDENT_/ set_authtype set_authflag get_authflag lookup query);
  
  $status = set_authtype("KERBEROS_V4","/etc/leland/ident.srvtab"); 

  $status = set_authflag("USER-INTERACTION","YES") ; 

  ($requester_port,$responder_port,$identifier,$opsys,$charset,$status) = 
	lookup(fileno(STDIN),$timeout) ; 

  ($requester_port,$responder_port,$identifier,$opsys,$charset,$status) = 
	query($responder_ip,$responder_port,
	      $requester_ip,$requester_port,$timeout) ; 



=head1 DESCRIPTION
  
	Net::Sident is a simple functional interface to the C sident 
library. The set_authtype call should be used before any attempts to
use any other part of the library. 

	There are two lookup methods. 

	lookup uses a file number and timeout as arguements.

	Query uses a dot quadded ip address and port number of 
both the requester and responder in addition to the timeout
parameter. Here's an an example

	query("127.0.0.1",25,"127.0.0.1",8000,30) ; 


=head1 Exported constants

  IDENT_AUTH_FAIL
  IDENT_AUTH_NOT_SUPPORTED
  IDENT_AUTH_OKAY
  IDENT_BUFFER_SIZE
  IDENT_FLAG_NOT_SUPPORTED
  IDENT_HIDDEN_USER
  IDENT_INTERNAL_ERR
  IDENT_INVALID_FLAG_VALUE
  IDENT_INVALID_PORT
  IDENT_INVALID_REQ_INFO
  IDENT_INVALID_RESP_INFO
  IDENT_MAX_ERROR
  IDENT_MUTUAL_AUTH_FAIL
  IDENT_NO_MUTUAL_AUTH
  IDENT_NO_USER
  IDENT_READ_TIMEOUT
  IDENT_SYSTEM_ERROR
  IDENT_TIMEOUT
  IDENT_UNKNOWN_ERROR
  IDENT_USER_CANT_AUTH
  IDENT_USER_WONT_AUTH

=head1 AUTHOR

Booker C. Bense : bbense@stanford.edu

=head1 SEE ALSO

perl(1),sident(3).

=cut
