/** @file
 * Simple but effective hack to test the kernel.o file.
 *
 * It takes on stdin the output from "netstat -f inet -n | grep ESTAB" in
 * either Solaris 2.x (non-standard formats can easily be converted to this)
 *
 *	laddr.lport faddr.fport .....
 *
 * or BSD 4.x (the defacto standard netstat output):
 *
 *	tcp <num> <num>  laddr.lport faddr.fport
 *
 * format.
 *
 * The output must be numeric, as non-numeric output is truncated when
 * hostnames get too long and ambiguous.  And we don't want netstat to first
 * convert numbers to names and then this program to convert names back to
 * numbers.
 *
 * @author Casper Dik (casper@fwi.uva.nl)
 *
 * $Id: itest.c,v 1.4 2002/10/29 00:52:15 eagle Exp $
 */

#include <ctype.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/socket.h>
#include <sys/time.h>
#include <sys/types.h>
#include <pwd.h>

/* BSDI needs <netinet/in.h> before <arpa/inet.h>. */
#include <netinet/in.h>
#include <arpa/inet.h>

#include "identd.h"
#include "error.h"
#include "pidentd.h"
#include "sident.h"

static int syslog_facility = LOG_DAEMON;

/*
 * To resolve external references that are usually resolved
 * from identd.c
 */
char *path_unix = NULL;
char *path_kmem = NULL;
int lport = 0;
int fport = 0;

int debug_flag = 1;
int syslog_flag = 1;

int
main()
{
    char buf[500];

    void *ka_ptr ; 
    struct  kernel *kinfo_ptr ; 


    /*
    ** Open the connection to the Syslog daemon if requested
    */
    if (syslog_flag)
    {
	openlog("identd_test", LOG_PID, syslog_facility);
    }


    if (ka_open(&ka_ptr) != 0) {
	fprintf(stderr,"ka_open failed\n");
	exit(1);
    }
    while (fgets(buf,sizeof(buf),stdin)) {
	char *loc, *rem, *tmp;
	unsigned short lport, fport;
	struct in_addr faddr, laddr;
	int uid;
	int pid;
	char *cmd, *cmd_and_args;

	struct passwd *pwd;
	char buf2[sizeof(buf)];

	strcpy(buf2,buf);

	loc = strtok(buf, " \t");
	if (strcmp(loc,"tcp") == 0) {
	    int i;
	    for (i = 0; i < 3; i++)
		loc = strtok(NULL, " \t");
	}
	rem = strtok(NULL, " \t");
	if (loc == NULL || rem == NULL) {
	    fprintf(stderr,"Malformed line: %s\n", buf2);
	    continue;
	}
	/* parse remote, local address */
	tmp = strrchr(loc,'.');
	if (tmp == NULL) {
	    fprintf(stderr,"Malformed line: %s\n", buf2);
	    continue;
	}
	*tmp++ ='\0';
	laddr.s_addr = inet_addr(loc);
	lport = atoi(tmp);

	tmp = strrchr(rem,'.');
	if (tmp == NULL) {
	    fprintf(stderr,"Malformed line: %s\n", buf2);
	    continue;
	}
	*tmp++ ='\0';
	fport = atoi(tmp);
	faddr.s_addr = inet_addr(rem);

	uid = -1;

	pid = 0;
	cmd = "unknown";
	cmd_and_args = "unknown";

	/* convert this to ka_lookup */ 


/*  	if (k_getuid(&faddr, htons(fport), &laddr, htons(lport), &uid */

/*  		, &pid */
/*  		, &cmd */
/*  		, &cmd_and_args */

/*  		) != 0) { */
/*  	    fprintf(stderr,"*unknown*\t%s\t%d\t\t%s\t%d\n", loc, lport, rem, fport); */
/*  	    continue; */
/*  	} */

	
  /* Fill up kernel info struct */ 

    /*  struct kernel */
    /*  { */
    /*       Request parameters  */
    /*      struct sockaddr_in local; */
    /*      struct sockaddr_in remote; */
    
    /*      Result parameters */
    /*      avail_t av; */
    
    /*      int status; */
    /*      uid_t euid; */
    /*      uid_t ruid; */
    
    /*      pid_t pid; */
    /*      char *cmd; */
    /*      char *argv; */
    /*  }; */

    kinfo_ptr = kernel_alloc() ; 


    kinfo_ptr->local.sin_port = htons(lport) ; 
    kinfo_ptr->remote.sin_port  = htons(fport) ;
    memcpy(&(kinfo_ptr->local.sin_addr),&laddr,sizeof(struct in_addr)); 
    memcpy(&(kinfo_ptr->remote.sin_addr),&faddr,sizeof(struct in_addr)); 
    kinfo_ptr->local.sin_family = AF_INET ; 
    kinfo_ptr->remote.sin_family = AF_INET ; 
    /* k_getuid returns 0 on success, any non-zero on failure. */

    
    if  ( ka_lookup(ka_ptr,kinfo_ptr) <  0){ 
      fprintf(stderr,"ka_lookup failed\n");
      exit(1);
    }
	


    pwd = getpwuid(kinfo_ptr->ruid);
    if (pwd)
      printf("%-8.8s", pwd->pw_name);
    else
      printf("%-8.8d", kinfo_ptr->ruid);

    printf (" \t%-13s\t%-4d\t%-13s\t%-4d\tPID=%d\tCMD=%s\tCMD+ARG=%s\n", loc, lport, rem, fport, kinfo_ptr->pid, kinfo_ptr->cmd, kinfo_ptr->argv);

    }

    return 0;
}
